<?php

/**
 *
 * @package plg_UP for Joomla! 3.0+
 * @version $Id: up.php 2017-07-02 $
 * @author Lomart
 * @copyright (c) 2017 Lomart
 * @license   <a href="http://www.gnu.org/licenses/gpl-3.0.html" target="_blank">GNU/GPLv3</a>
 *
 * */
//namespace up;
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Version;

class plgContentUP extends JPlugin {

    public $upPath = 'plugins/content/up/';

    public function __construct(&$subject, $params) {
        parent::__construct($subject, $params);
        $this->LoadLanguage();
    }

    function onContentPrepare($context, &$article, &$params, $limitstart = 0) {
        $app = JFactory::getApplication();

        if ($context == 'com_finder.indexer') {
            // les identificateurs autorisés pour UP
            $tags = $this->params->def('tagname', 'up|xx');
            // les identificateurs des shortcodes internes (marker, default-area, ...)
            // ils sont à déclarer par l'action (TODO)
            $internal_tags = 'marker|default-area';
            // les actions pour lesquelles il est dangereux de montrer le contenu
            // ils sont à déclarer par l'action (TODO)
            $ActionMaskedContent = 'note|filter';
            
            // 1 - effacer les shortcodes et contenus des actions confidentielles
            $regex = '#(\{(?:'.$tags.')\s*(?:'.$ActionMaskedContent.').*\{\/(?:up|xx)\s*(?:'.$ActionMaskedContent.').*\}s)#Ui';
            $article->text = preg_replace($regex, '', $article->text);
                
            // 2 - masquer les shortcodes ouvrants et/ou uniques
            $regex = '#(\{(?:'.$tags.'|'.$internal_tags.'|===).*\})#Ui';
            $article->text = preg_replace($regex, '', $article->text);
            
            // 2 - masquer les shortcodes
            $regex = '#(\{\/(?:'.$tags.').*\})#Ui';
            $article->text = preg_replace($regex, '', $article->text);
            
            return;
        }
        
        // ========> DOIT-ON EXECUTER ?
        // uniquement en frontend
        if ($app->isClient('administrator'))
            return false;

        // Chargement systematique de la feuile de style
        if ($this->params->def('loadcss', '1')) {
            JHtml::stylesheet('plugins/content/up/assets/up.css');
        }

        // sortie directe si pas de texte a traiter
        if (!isset($article->text)) {
            return false;
        }
        if (trim($article->text) == '') {
            return false;
        }
        // pas d'analyse pour les listes d'articles
        if ($context == "com_content.category") {
            if ($app->input->get('layout', 0) !== 'blog') {
                return false;
            }
        }
        // liste des shortcodes utilisables
        $tag = $this->params->def('tagname', 'up|xx');
        $regexopen = '/\{(?:' . $tag . ') *([^\s\=\|\{\}]+)/si';
        // retour direct si pas de upAction dans l'article
        if (!preg_match($regexopen, $article->text))
            return false;

        // ==========> C'EST BON, IL FAUT Y ALLER !
        // fonctions utilitaires pour les actions
        include_once $this->upPath . 'upAction.php';

        // charger le dictionnaire
        $dico = file_get_contents('dico.json', FILE_USE_INCLUDE_PATH);
        $dico = json_decode($dico, true);

        /*         * ****** PSEUDO-CODE
         * 0. doit-on supprimer les <p...>{up et }</p> ?
         * 1. on récupère la positions de tous les "{up " dans $article->text
         *    => $openSC[$i][actionName] : nom de la l'action
         *    => $openSC[$i][posDeb] : position de l'accolade ouvrante
         * 2. on parcours les shortcodes à partir du dernier
         *    la fin du shortcode ouvrant est le 1er } suivant {
         *    (note: les accolades des shortcodes enfants n'existent plus car traitées en premier)
         *    $actionUsername =  $openSC[$i][actionName]
         *    $replaceDeb = $openSC[$i]['posDeb'];
         *    $replaceLen = taille shortcode ouvrant {} inclus
         * 3. on analyse les options en nettoyant les <p> et </br>
         *    note: les shortcodes inclus ne doivent pas renvoyer de <p> et </br> (ou les mettre en [p] et [br])
         * 4. on recherche un shortcode fermant et on actualise $replaceDeb & $replaceLen
         *
         */

        // Pour éviter nettoyage par editeur wysiwyg, on utilise un simili BBCode dans les arguments options
        $bbcode = array('[br]');
        $htmlcode = array('<br>');

        // ===== NETTOYAGE AJOUT EDITEURS (v1.8)
        // on supprime les balises P vides limitrophes ajoute par les editeurs wysiwyg
        $regex = '#(<p>)(\{\/?' . $tag . '\b.*\})(</p>)#U';
        $article->text = preg_replace($regex, '$2', $article->text);

        // ===== RECHERCHE DE TOUS LES SHORTCODES OUVRANTS
        if (!preg_match_all($regexopen, $article->text, $matches, PREG_OFFSET_CAPTURE)) {
            $this->info_debug('Error up.php lign 91');
        }
        $nbSC = count($matches[0]);
        for ($i = 0; $i < $nbSC; $i++) {
            $openSC[$i]['actionName'] = trim($matches[1][$i][0]);
            $openSC[$i]['posDeb'] = $matches[0][$i][1];
        }

        // liste des objets ACTION initialisés
        $classObjList = array();

        // ==== parcours des shortcodes ouvrants a partir du dernier
        for ($i = $nbSC - 1; $i >= 0; $i--) {
            // reset variables
            unset($actionUserName);  // nom de l'action saisi dans le shortcode
            unset($actionClassName); // nom du dossier, script php et classe de l'action
            unset($options_user);
            $content = '';  // le contenu entre shortcodes
            unset($ret); // retour par action
            // identifiant unique pour l'action
            if (isset($article->id)) {  // si article
                $options_user['id'] = 'up-' . $article->id . '-' . ($i + 1);
            } else { // si module
                $options_user['id'] = 'up-' . uniqid(); //v2.9
            }

            // -- Le shortcode ouvrant complet : {up action=xx | opt=val}
            $SC = strstr(substr($article->text, $openSC[$i]['posDeb']), '}', true) . '}';

            // -- position pour remplacement au retour
            $replaceDeb = $openSC[$i]['posDeb'];
            $replaceLen = strlen($SC);
            // -- Les options du shortcode ouvrant : action=xx | opt=val
            // 29/9/19 pour shortcode multilignes en wysiwyg
            // <br /> est le saut de ligne utilise par LM-Prism, tiny et JCE (exemple démo)
            $SC = str_ireplace(array('<p>', '</p>', '<br />', '&nbsp;', PHP_EOL, '{', '}'), '', $SC);
            $SC = substr($SC, strpos($SC, ' '));
//			// -- analyse des options du shortcode
            $allParams = explode('|', $SC);
            foreach ($allParams as $param) {
                // v1.8 supprime espace dur de TinyMCE
                $param = preg_split("/=/", trim($param, " \t\n\r\0\x0B\xA0\xC2"), 2); // permet = dans argument
                // le mot clé tel que saisi
                $key = strtolower(trim($param[0]));
                // sa valeur (true si aucune)
                $value = (count($param) == 2) ? trim($param[1]) : true;
                // suppression d'un saut de ligne <br> ou <br /> entre les options
                /* $value = preg_replace('#\s*\<br\s?\/?>#i', '', $value); */
                // la 1ere option est le nom de l'action
                if (!isset($actionUserName)) {
                    $actionUserName = $key;  // tel que saisi dans article
                    //LM 180921: l'argument principal est égal à vide et pas true
                    $value = (count($param) == 2) ? trim($param[1]) : '';
                    // le mot clé traduit pour le script action
                    $key = str_replace('-', '_', $key); // tel que le script (14/12/19)
                    if (array_key_exists($key, $dico)) {
                        $key = $dico[$key];
                    }
                    $key = str_replace('-', '_', $key); // tel que le script
                    $actionClassName = $key;  // Nom dossier et classe
                } else {
                    // le mot clé traduit pour le script action
                    if (array_key_exists($key, $dico)) {
                        $key = $dico[$key];
                    }
                }
                // remplacement simili-BBCode
                $value = str_replace($bbcode, $htmlcode, $value);
                // analyse de l'argument de l'option
                $options_user[$key] = $value;
            }
            // -- on recherche l'eventuel shortcode fermant
            $regexclose = '/\{\/(?:' . $tag . ')\s+' . $actionUserName . '.*\}/siU';
            if (preg_match($regexclose, $article->text, $matches, PREG_OFFSET_CAPTURE, $replaceDeb + $replaceLen)) {
                // le contenu
                $content_deb = $replaceDeb + $replaceLen;
                $content_len = $matches[0][1] - $content_deb;
                $content = substr($article->text, $content_deb, $content_len);

                // suppression balise P fermante au début et ouvrante à la fin
                // <p>{shortcode}</p>contenu<p>{/shortcode}</p>
                $regex = array('#^</.*>#', '#<[a-zA-Z =-_"]*>$#U');
                $content = preg_replace($regex, '', $content);
                // maj positions remplacement
                $replaceLen = $replaceLen + $content_len + strlen($matches[0][0]);
            }
            // ==== EXECUTION DE L'ACTION
            $text = '';
            // le chemin du script
            $actionfile = 'actions/' . $actionClassName . '/' . $actionClassName . '.php';

            // --- instanciation de l'action
            // si premier appel de l'action
            if ($text == '') {
                if (array_key_exists($actionClassName, $classObjList) == false) {
                    // on charge la classe de l'action
                    if (@include_once $actionfile) {
                        $classObjList[$actionClassName] = new $actionClassName($actionClassName);
                        $classObjList[$actionClassName]->actionUserName = $actionUserName;
                        $classObjList[$actionClassName]->firstInstance = true; // pour action unique par page dans run
                        $classObjList[$actionClassName]->article = $article; // pour load_js_file_head
                        $classObjList[$actionClassName]->init();
						$objVersion = new Version; // v2.6
						$classObjList[$actionClassName]->J4 = ((int)$objVersion->getShortVersion() >= 4);
                    } else {
                        $msg = ($actionUserName=='') ? 'Syntax error' : 'non trouvée / not found'; //v2.7
                        $text = '&#x1F199; ' . $options_user['id'] . ' Action "<b>' . $openSC[$i]['actionName'] . '</b>" ' .$msg;
                        $app = JFactory::getApplication();
                        $app->enqueueMessage($text, 'error');
                    }
                } else {
                    $classObjList[$actionClassName] = new $actionClassName($actionClassName);
                    $classObjList[$actionClassName]->actionUserName = $actionUserName;
                    $classObjList[$actionClassName]->firstInstance = false; // 07-18:pour action unique par page dans run
                }
            }

            if ($text == '') {
                // l'objet est cree et initialisé
                $classObjList[$actionClassName]->options_user = $options_user;
                $classObjList[$actionClassName]->content = $content;
                $classObjList[$actionClassName]->article = $article;
                $classObjList[$actionClassName]->actionprefs = $this->params->get('actionprefs');
                $classObjList[$actionClassName]->usehelpsite = $this->params->get('usehelpsite', '2');
                $classObjList[$actionClassName]->urlhelpsite = $this->params->get('urlhelpsite');
                $classObjList[$actionClassName]->tarteaucitron = $this->params->def('tarteaucitron', false); // v2.4
                $classObjList[$actionClassName]->demopage = '';
                $classObjList[$actionClassName]->dico = $dico;
                // 18-07-20 ajout pour remplacement par action
                $classObjList[$actionClassName]->replace_deb = $replaceDeb;
                $classObjList[$actionClassName]->replace_len = $replaceLen;
                // on exécute l'action
                $ret = $classObjList[$actionClassName]->run();
            }

            $ret = (isset($ret)) ? $ret : '';
            if (!is_array($ret)) {
                // texte pour remplacement (méthode originelle)
                // on remplace le shortcode par le code retourné par l'action
                $article->text = substr_replace($article->text, $ret, $replaceDeb, $replaceLen);
            } else {
                if (isset($ret['all'])) {
                    // l'action a traité l'intégralité des remplacements (cas action TOC)
                    $article->text = $ret['all'];
                }
                if (isset($ret['tag'])) {
                    // remplacement du shortcode
                    $article->text = substr_replace($article->text, $ret['tag'], $replaceDeb, $replaceLen);
                }
                // ajout en début d'article
                if (isset($ret['before'])) {
                    $article->text = $ret['before'] . $article->text;
                }
                // ajout en fin d'article
                if (isset($ret['after'])) {
                    $article->text = $article->text . $ret['after'];
                }
            }
        } // fin parcours openSC

        unset($classObjList);
        return true;
    }

// onContentPrepare

    /* ==== lang
     * fonction utilitaire pour UP
     * @param  [string] $str [alternative de traduction sous la forme "en=apple;fr=pomme"]
     * @return [string]      [la traduction dans la langue]
     */
    function lang($str) {
        // l'argument doit faire au minimum 10 caractères (fr=xx;en=xx)
        $out = trim($str);
        if (strlen($out) <= 10)
            return $str;

        // -- v1.6 : permettre l'arg commencant par lang[
        if (substr(strtolower($out), 0, 5) == 'lang[')
            $out = substr($out, 5, -1);

        // test langue uniquement sur les 2 premiers caractères
        $codelang = substr(JFactory::getLanguage()->getTag(), 0, 2);

        // recherche du motif dans $str. Il faut au moins 2 langues
        if (preg_match_all('#(\w\w) *= *(.*);#U', $out . ';', $tmp) > 1) {
            if (isset($tmp[0][1])) {
                $trad = array_combine($tmp[1], $tmp[2]);
                if (isset($trad[$codelang])) {
                    $out = $trad[$codelang];   // dans la langue
                } elseif (isset($trad['en'])) {
                    $out = $trad['en']; // sinon en anglais
                } elseif ($str[2] == '=') {
                    $out = $trad[$tmp[1][0]]; // sinon le premier
                    //v1.8 - retour totalité car pb si url du type : index.php?option=com_content&amp;id=...
                    //v1.9 - on retourne le 1er si le 3e caractère est le signe égal
                }
                $out = trim($out);
            }
        }
        return $out;
    }

    /* ==== info_debug
     * utilisé pour indiquer une erreur à son emplacement dans la page
     * $txt accepte la forme : en:hello;fr:bonjour
     * exemple : argument de paramètre manquant
     */

    function info_debug($txt, $infoUP = true) {
        $txt = $this->lang($txt);
        if ($infoUP)
            $txt = 'UP.' . $this->actionUserName . ' : ' . $txt;
        return ' <span style="color:red;background:yellow;font-weight:bolder"> &#x279c; ' . $txt . '&nbsp;</span>';
    }

    /* ==== onAjaxUp
     * appels AJAX pour toutes les actions
     */

    function onAjaxUp() {
        $input = JFactory::getApplication()->input;
        $data = $input->get('data', '', 'string');
        parse_str($data, $output);
        if (!isset($output['action'])) {
            return 'err : action incorrect';
        }
        $actionClassName = $output['action'];
        $actionfile = $this->upPath . 'actions/' . $actionClassName . '/ajax_' . $actionClassName . '.php';

        if (@include_once $actionfile) {
            return $actionClassName::goAjax($input);
        } else {
            $text = 'Action Ajax ' . $actionClassName . ' non trouvée / not found';
            return 'err : ' . $text;
        }
    }

}

// class


